function [p,e]=lower_bound(S,Sg1,Exp1,n,m,u)

%   This function uses a modification of an algorithm in the paper
%      "Numerical Computation of Multivariate Normal Probabilities", 
%      A. Genz, J. of Comp. Graph. Stat., 1(1992), pp. 141-149.
%
%   The primary references for the numerical integration are 
%    "Randomization of Number Theoretic Methods for Multiple Integration",
%     R. Cranley & T.N.L. Patterson, SIAM J Numer Anal, 13(1976), pp. 904-14.
%   and  
%    "Fast Component-by-Component Construction, a Reprise for Different 
%     Kernels", D. Nuyens & R. Cools. In H. Niederreiter and D. Talay,
%     editors, Monte-Carlo and Quasi-Monte Carlo Methods 2004, 
%     Springer-Verlag, 2006, 371-385.


        m = 5*m;
        eps1 = 1e-9;
        [ U, D ] = svd(Sg1(1:n+1,1:n+1));
        d = sqrt(diag(D));
        nc =n+1;
        if d(nc) < eps1, 
            nc = min( find( d < eps1 ) );
        end
        [ Q, R ] = qr( diag(d(1:nc))*U(:,1:nc)' );
        ch = ( diag(sign(diag(R)))*R )'; 
if nc > 64, 
    q = sqrt(primes(5*(nc)*log(nc+1)/4))'; % Richtmyer generators
else
    [ q m ] = fstrnk( m, nc-1 ); 
    q = q/m; % Lattice rule
end

y = zeros(nc-1,m);
df = zeros(1,S);
p = 0;
e = 0;

for K = 1 : S,  
    vp = Phi2((u-Exp1(1))/ch(1,1));
    bi = 9; 
    ai = -9;
  %
  % Compute randomized MVN integrand with tent periodization
  %
  y(1,:)= Phinv(abs( 2*mod(q(1)*[1:m]+rand,1) - 1 ).*vp );
  for i = 2 : nc-1
    ys = ch(i,1:i-1)*y(1:i-1,:); 
    dc = Phi2( min( ( u - Exp1(i) - ys )/ch(i,i), 9 ) ); 
    vp = vp.*dc; 
    y(i,:) = Phinv(abs( 2*mod(q(i)*[1:m]+rand,1) - 1 ).*dc ); 
  end
  for i = nc : n+1, 
      ys = ch(i,1:nc-1)*y(1:nc-1,:);
    if ch(i,nc) > 0
      bi = min( ( u - Exp1(i) - ys )/ch(i,nc), bi );  
    else
      ai = max( ( u - Exp1(i) - ys )/ch(i,nc), ai );  
    end
  end
  df(K) = ( mean( vp.*( Phi2(max(ai,bi)) - Phi2(min(ai,bi)) ) )); p = p +df(K); 
end
p = p /S; 
e = 1/(S*(S-1))*sum((df-ones(1,S)*p).^2);
e = 3*sqrt(e); % error estimate is 3 x standard error with n samples.
p = 1 - p;
